@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <string>
 #include <string.h>
@@ -45,10 +45,6 @@
 #include "PView.h"
 #endif
 
-#if defined(HAVE_MESH)
-#include "periodical.h"
-#endif
-
 #if defined(HAVE_PARSER)
 #include "Parser.h"
 #endif
@@ -78,19 +74,22 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-o file",            "Specify output file name"));
   s.push_back(mp("-format string",     "Select output mesh format (auto (default), msh, "
                                        "msh1, msh2, unv, vrml, ply2, stl, mesh, bdf, cgns, "
-                                       "p3d, diff, med, ...)"));
+                                       "p3d, diff, med, neu, ...)"));
   s.push_back(mp("-bin",               "Use binary format when available"));
   s.push_back(mp("-refine",            "Perform uniform mesh refinement, then exit"));
+  s.push_back(mp("-reclassify",        "Reclassify mesh, then exit"));
   s.push_back(mp("-part int",          "Partition after batch mesh generation"));
   s.push_back(mp("-partWeight tri|quad|tet|prism|hex int", "Weight of a triangle/quad/etc. "
                                                            "during partitioning"));
-  s.push_back(mp("-saveall",           "Save all elements (discard physical group definitions)"));
-  s.push_back(mp("-parametric",        "Save vertices with their parametric coordinates"));
+  s.push_back(mp("-save_all",          "Save all elements (discard physical group definitions)"));
+  s.push_back(mp("-save_parametric",   "Save vertices with their parametric coordinates"));
+  s.push_back(mp("-save_topology",     "Save model topology"));
   s.push_back(mp("-algo string",       "Select mesh algorithm (meshadapt, del2d, front2d, "
                                         "delquad, del3d, front3d, mmg3d, pack)"));
   s.push_back(mp("-smooth int",        "Set number of mesh smoothing steps"));
   s.push_back(mp("-order int",         "Set mesh order (1, ..., 5)"));
   s.push_back(mp("-optimize[_netgen]", "Optimize quality of tetrahedral elements"));
+  s.push_back(mp("-optimize_threshold", "Optimize tetrahedral elements that have a qulaity less than a threshold"));
   s.push_back(mp("-optimize_ho",       "Optimize high order meshes"));
   s.push_back(mp("-ho_[min,max,nlayers]", "High-order optimization parameters"));
   s.push_back(mp("-optimize_lloyd",    "Optimize 2D meshes using Lloyd algorithm"));
@@ -107,7 +106,9 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-rand float",        "Set random perturbation factor"));
   s.push_back(mp("-bgm file",          "Load background mesh from file"));
   s.push_back(mp("-check",             "Perform various consistency checks on mesh"));
-  s.push_back(mp("-ignorePartBound",   "Ignore partitions boundaries"));
+  s.push_back(mp("-ignorePartBound",   "Ignore partition boundaries"));
+  s.push_back(mp("-ignorePeriocity",   "Ignore periodic boundaries"));
+  s.push_back(mp("-oneFilePerPart",    "Save mesh partitions in separate files"));
 #if defined(HAVE_FLTK)
   s.push_back(mp("Post-processing options:", ""));
   s.push_back(mp("-link int",          "Select link mode between views (0, 1, 2, 3, 4)"));
@@ -149,6 +150,8 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-setstring name value", "Set constant string name=value"));
   s.push_back(mp("-option file",       "Parse option file at startup"));
   s.push_back(mp("-convert files",     "Convert files into latest binary formats, then exit"));
+  s.push_back(mp("-nt int",            "Set number of threads"));
+  s.push_back(mp("-cpu",               "Report CPU times for all operations"));
   s.push_back(mp("-version",           "Show version number"));
   s.push_back(mp("-info",              "Show detailed version information"));
   s.push_back(mp("-help",              "Show command line usage"));
@@ -175,10 +178,15 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("2 or F2",        "Mesh surfaces"));
   s.push_back(mp("3 or F3",        "Mesh volumes"));
   s.push_back(mp("Escape",         "Cancel lasso zoom/selection, toggle mouse selection ON/OFF"));
+  s.push_back(mp("e",              "End/accept selection in geometry creation mode"));
   s.push_back(mp("g",              "Go to geometry module"));
   s.push_back(mp("m",              "Go to mesh module"));
   s.push_back(mp("p",              "Go to post-processing module"));
+  s.push_back(mp("q",              "Abort selection in geometry creation mode"));
   s.push_back(mp("s",              "Go to solver module"));
+  s.push_back(mp("x",              "Toogle x coordinate freeze in geometry creation mode"));
+  s.push_back(mp("y",              "Toogle y coordinate freeze in geometry creation mode"));
+  s.push_back(mp("z",              "Toogle z coordinate freeze in geometry creation mode"));
   s.push_back(mp("Shift+a",        "Bring all windows to front"));
   s.push_back(mp("Shift+g",        "Show geometry options"));
   s.push_back(mp("Shift+m",        "Show mesh options"));
@@ -187,8 +195,12 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("Shift+s",        "Show solver options"));
   s.push_back(mp("Shift+u",        "Show post-processing view plugins"));
   s.push_back(mp("Shift+w",        "Show post-processing view options"));
+  s.push_back(mp("Shift+x",        "Move only along x coordinate in geometry creation mode"));
+  s.push_back(mp("Shift+y",        "Move only along y coordinate in geometry creation mode"));
+  s.push_back(mp("Shift+z",        "Move only along z coordinate in geometry creation mode"));
   s.push_back(mp("Shift+Escape",   "Enable full mouse selection"));
   s.push_back(mp(cc + "d",         "Attach/detach menu"));
+  s.push_back(mp(cc + "e",         "Export project"));
   s.push_back(mp(cc + "f",         "Enter full screen"));
   s.push_back(mp(cc + "i",         "Show statistics window"));
   s.push_back(mp(cc + "j",         "Save model options"));
@@ -200,14 +212,13 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp(cc + "o",         "Open project file"));
   s.push_back(mp(cc + "q",         "Quit"));
   s.push_back(mp(cc + "r",         "Rename project file"));
-  s.push_back(mp(cc + "s",         "Save file as"));
+  s.push_back(mp(cc + "s",         "Save mesh in default format"));
   s.push_back(mp("Shift+" + cc + "c", "Show clipping plane window"));
   s.push_back(mp("Shift+" + cc + "h", "Show current options and workspace window"));
   s.push_back(mp("Shift+" + cc + "j", "Save options as default"));
   s.push_back(mp("Shift+" + cc + "m", "Show manipulator window"));
   s.push_back(mp("Shift+" + cc + "n", "Show option window"));
   s.push_back(mp("Shift+" + cc + "o", "Merge file(s)"));
-  s.push_back(mp("Shift+" + cc + "s", "Save mesh in default format"));
   s.push_back(mp("Shift+" + cc + "u", "Show plugin window"));
   s.push_back(mp("Shift+" + cc + "v", "Show visibility window"));
   s.push_back(mp("Alt+a",          "Loop through axes modes"));
@@ -232,13 +243,13 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("Alt+z",          "Set Z view"));
   s.push_back(mp("Alt+Shift+a",    "Hide/show small axes"));
   s.push_back(mp("Alt+Shift+b",    "Hide/show mesh volume faces"));
+  s.push_back(mp("Alt+Shift+c",    "Loop through predefined colormaps"));
   s.push_back(mp("Alt+Shift+d",    "Hide/show mesh surface faces"));
   s.push_back(mp("Alt+Shift+l",    "Hide/show mesh lines"));
   s.push_back(mp("Alt+Shift+p",    "Hide/show mesh points"));
   s.push_back(mp("Alt+Shift+s",    "Hide/show mesh surface edges"));
   s.push_back(mp("Alt+Shift+t",    "Same as Alt+t, but with numeric mode included"));
   s.push_back(mp("Alt+Shift+v",    "Hide/show mesh volume edges"));
-  s.push_back(mp("Alt+Shift+w",    "Reverse all mesh normals"));
   s.push_back(mp("Alt+Shift+x",    "Set -X view"));
   s.push_back(mp("Alt+Shift+y",    "Set -Y view"));
   s.push_back(mp("Alt+Shift+z",    "Set -Z view"));
@@ -284,9 +295,10 @@ void PrintUsage(const std::string &name)
 
 void GetOptions(int argc, char *argv[])
 {
-  // print messages on terminal
+  // print messages on terminal (use special 99 value so that we can detect if
+  // it was later set to 1 in the option file)
   int terminal = CTX::instance()->terminal;
-  CTX::instance()->terminal = 1;
+  CTX::instance()->terminal = 99;
 
 #if defined(HAVE_PARSER)
   if(argc && argv){
@@ -362,10 +374,22 @@ void GetOptions(int argc, char *argv[])
         CTX::instance()->batch = 4;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "cpu")) {
+        Msg::SetInfoCpu(true);
+        i++;
+      }
+      else if(!strcmp(argv[i] + 1, "log")) {
+        Msg::SetLogFile("gmsh.log");
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "refine")) {
         CTX::instance()->batch = 5;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "reclassify")) {
+        CTX::instance()->batch = 6;
+        i++;
+      }
       else if(!strcmp(argv[i] + 1, "renumber")) {
         CTX::instance()->batchAfterMesh = 1;
         CTX::instance()->partitionOptions.renumber = 1;
@@ -407,6 +431,10 @@ void GetOptions(int argc, char *argv[])
               i++;
               opt_mesh_partition_pyr_weight(0,GMSH_SET,atoi(argv[i]));
             }
+            else if (!strcmp(argv[i],"trihedron")) {
+              i++;
+              opt_mesh_partition_trih_weight(0,GMSH_SET,atoi(argv[i]));
+            }
             else if (!strcmp(argv[i],"hex")) {
               i++;
               opt_mesh_partition_hex_weight(0,GMSH_SET,atoi(argv[i]));
@@ -466,9 +494,21 @@ void GetOptions(int argc, char *argv[])
         i++;
       }
       else if(!strcmp(argv[i] + 1, "optimize")) {
+	Msg::Warning("The '-optimize' option is now obsolete: "
+                     "Gmsh optimizes tetrahedral meshes by default");
+	Msg::Warning("Use '-optimize_threshold threshold' to "
+                     "control which elements are optimized");
+	Msg::Warning("Option '-optimize_threshold 0' leads to no optimization");
         CTX::instance()->mesh.optimize = 1;
         i++;
       }
+      else if(!strcmp(argv[i] + 1, "optimize_threshold")) {
+        i++;
+        if(argv[i])
+          opt_mesh_optimize_threshold(0, GMSH_SET, atof(argv[i++]));
+        else
+          Msg::Fatal("Missing number");
+      }
       else if(!strcmp(argv[i] + 1, "optimize_netgen")) {
         CTX::instance()->mesh.optimizeNetgen = 1;
         i++;
@@ -506,23 +546,6 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing number of lloyd iterations");
       }
-      else if(!strcmp(argv[i] + 1, "bunin")) {
-        i++;
-        if(argv[i])
-          CTX::instance()->mesh.bunin = atoi(argv[i++]);
-        else
-          Msg::Fatal("Missing cavity size in bunin optimization");
-      }
-#if defined(HAVE_MESH)
-      else if(!strcmp(argv[i] + 1, "microstructure")) {
-        i++;
-        if(argv[i]) microstructure(argv[i++]);
-      }
-      else if(!strcmp(argv[i] + 1, "computeBestSeeds")) {
-        i++;
-        if(argv[i]) computeBestSeeds(argv[i++]);
-      }
-#endif
       else if(!strcmp(argv[i] + 1, "nopopup")) {
         CTX::instance()->noPopup = 1;
         i++;
@@ -550,7 +573,7 @@ void GetOptions(int argc, char *argv[])
       else if(!strcmp(argv[i] + 1, "setstring")) {
         i++;
 	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
-          gmsh_yystringsymbols[argv[i]] = argv[i + 1];
+          gmsh_yystringsymbols[argv[i]] = std::vector<std::string>(1, argv[i + 1]);
           Msg::GetCommandLineStrings()[argv[i]] = argv[i + 1];
           i += 2;
 	}
@@ -623,14 +646,14 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing file name");
       }
-      else if(!strcmp(argv[i] + 1, "nw")) {
+      else if(!strcmp(argv[i] + 1, "windows")) {
         i++;
         if(argv[i])
           CTX::instance()->numWindows = atoi(argv[i++]);
         else
           Msg::Fatal("Missing number");
       }
-      else if(!strcmp(argv[i] + 1, "nt")) {
+      else if(!strcmp(argv[i] + 1, "tiles")) {
         i++;
         if(argv[i])
           CTX::instance()->numTiles = atoi(argv[i++]);
@@ -661,7 +684,7 @@ void GetOptions(int argc, char *argv[])
 #endif
           // convert mesh to latest binary format
           if(GModel::current()->getMeshStatus() > 0){
-            CTX::instance()->mesh.mshFileVersion = 2.0;
+            CTX::instance()->mesh.mshFileVersion = 3.0;
             CTX::instance()->mesh.binary = 1;
             CreateOutputFile(fileName, FORMAT_MSH);
           }
@@ -735,6 +758,14 @@ void GetOptions(int argc, char *argv[])
         i++;
         opt_mesh_ignore_part_bound(0, GMSH_SET, 1);
       }
+      else if(!strcmp(argv[i] + 1, "oneFilePerPart")) {
+        i++;
+        opt_mesh_msh_file_partitioned(0, GMSH_SET, 1);
+      }
+      else if(!strcmp(argv[i] + 1, "ignorePeriodicity")) {
+        i++;
+        opt_mesh_ignore_periodicity(0, GMSH_SET, 1);
+      }
       else if(!strcmp(argv[i] + 1, "edgelmin")) {
         i++;
         if(argv[i]) {
@@ -815,10 +846,16 @@ void GetOptions(int argc, char *argv[])
         i++;
         CTX::instance()->mesh.binary = 1;
       }
-      else if(!strcmp(argv[i] + 1, "parametric")) {
+      else if(!strcmp(argv[i] + 1, "save_parametric") ||
+              !strcmp(argv[i] + 1, "parametric")) {
         i++;
         CTX::instance()->mesh.saveParametric = 1;
       }
+      else if(!strcmp(argv[i] + 1, "save_topology") ||
+              !strcmp(argv[i] + 1, "save_entities")) {
+        i++;
+        CTX::instance()->mesh.saveTopology = 1;
+      }
       else if(!strcmp(argv[i] + 1, "algo")) {
         i++;
         if(argv[i]) {
@@ -838,6 +875,8 @@ void GetOptions(int argc, char *argv[])
             CTX::instance()->mesh.algo2d = ALGO_2D_FRONTAL;
           else if(!strncmp(argv[i], "bamg",4))
             CTX::instance()->mesh.algo2d = ALGO_2D_BAMG;
+          else if(!strncmp(argv[i], "gmsh3d", 6))
+            CTX::instance()->mesh.algo3d = ALGO_3D_DELAUNAY_NEW;
           else if(!strncmp(argv[i], "del3d", 5) || !strncmp(argv[i], "tetgen", 6))
             CTX::instance()->mesh.algo3d = ALGO_3D_DELAUNAY;
           else if(!strncmp(argv[i], "front3d", 7) || !strncmp(argv[i], "netgen", 6))
@@ -876,9 +915,9 @@ void GetOptions(int argc, char *argv[])
         i++;
         CTX::instance()->mesh.recombinationTestNoGreedyStrat = 1;
       }
-      else if(!strcmp(argv[i] + 1, "newstrat")) {
+      else if(!strcmp(argv[i] + 1, "meshdiscrete")) {
         i++;
-        CTX::instance()->mesh.recombinationTestNewStrat = 1;
+        CTX::instance()->meshDiscrete = 1;
       }
       else if(!strcmp(argv[i] + 1, "format") || !strcmp(argv[i] + 1, "f")) {
         i++;
@@ -913,22 +952,7 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "listen")) {
         i++;
-#if !defined(HAVE_ONELAB2)
         CTX::instance()->solver.listen = 1;
-#else
-        if(argv[i]) {
-          if(strstr(argv[i], "/") || strstr(argv[i], "\\") || !strstr(argv[i], ":")) // UNIX
-            CTX::instance()->onelab.unixSock = std::string(argv[i]);
-          else if(argv[i][0] == 'u') // UDT
-            CTX::instance()->onelab.udtSock = std::string(argv[i++]);
-          else if(argv[i][0] == 't') // TCP
-            CTX::instance()->onelab.tcpSock = std::string(argv[i++]);
-          else // TCP
-            CTX::instance()->onelab.tcpSock = std::string(argv[i]);
-        }
-        else // TCP
-          CTX::instance()->onelab.tcpSock = "127.0.0.1:1148";
-#endif
       }
       else if(!strcmp(argv[i] + 1, "minterpreter")) {
         i++;
@@ -985,13 +1009,13 @@ void GetOptions(int argc, char *argv[])
                 MED_NUM_MINEUR, MED_NUM_RELEASE);
 #endif
         fprintf(stderr, "Packaged by      : %s\n", GMSH_PACKAGER);
-        fprintf(stderr, "Web site         : http://www.geuz.org/gmsh/\n");
-        fprintf(stderr, "Mailing list     : gmsh@geuz.org\n");
+        fprintf(stderr, "Web site         : http://gmsh.info\n");
+        fprintf(stderr, "Mailing list     : gmsh@onelab.info\n");
         Msg::Exit(0);
       }
       else if(!strcmp(argv[i] + 1, "help") || !strcmp(argv[i] + 1, "-help")) {
         fprintf(stderr, "Gmsh, a 3D mesh generator with pre- and post-processing facilities\n");
-        fprintf(stderr, "Copyright (C) 1997-2015 Christophe Geuzaine and Jean-Francois Remacle\n");
+        fprintf(stderr, "Copyright (C) 1997-2017 Christophe Geuzaine and Jean-Francois Remacle\n");
         PrintUsage(argv[0]);
         Msg::Exit(0);
       }
@@ -1002,6 +1026,13 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing number");
       }
+      else if(!strcmp(argv[i] + 1, "nt")) {
+        i++;
+        if(argv[i])
+          opt_general_num_threads(0, GMSH_SET, atoi(argv[i++]));
+        else
+          Msg::Fatal("Missing number");
+      }
 #if defined(HAVE_FLTK)
       else if(!strcmp(argv[i] + 1, "term")) {
         terminal = 1;
@@ -1064,8 +1095,6 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "stereo")) {
         opt_general_stereo_mode(0, GMSH_SET, 1.);
-	//        CTX::instance()->camera = 1;
-	//	CTX::instance()->stereo = 1;
         i++;
       }
       else if(!strcmp(argv[i] + 1, "gamepad")) {
@@ -1132,5 +1161,6 @@ void GetOptions(int argc, char *argv[])
   else
     GModel::current()->setFileName(CTX::instance()->files[0]);
 
-  CTX::instance()->terminal = terminal;
+  if(CTX::instance()->terminal == 99)
+    CTX::instance()->terminal = terminal;
 }
