@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2017 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <sstream>
 #include <iomanip>
@@ -17,6 +17,7 @@
 #include "MHexahedron.h"
 #include "MPrism.h"
 #include "MPyramid.h"
+#include "MTrihedron.h"
 #include "StringUtils.h"
 #include "discreteVertex.h"
 #include "discreteEdge.h"
@@ -74,13 +75,15 @@ static void readMSHEntities(FILE *fp, GModel *gm)
     GFace *gf = gm->getFaceByTag(tag);
     if (!gf){
       discreteFace *df = new discreteFace(gm, tag);
-      std::vector<int> edges;
+      std::vector<int> edges, signs;
       for (int j = 0; j < n; j++){
 	int tage;
 	if(fscanf(fp, "%d", &tage) != 1) return;
-	edges.push_back(tage);
+	edges.push_back(std::abs(tage));
+        int sign = tage > 0 ? 1 : -1;
+        signs.push_back(sign);
       }
-      df->setBoundEdges(gm, edges);
+      df->setBoundEdges(edges, signs);
       gm->add(df);
       gf = df;
     }
@@ -92,13 +95,15 @@ static void readMSHEntities(FILE *fp, GModel *gm)
     GRegion *gr = gm->getRegionByTag(tag);
     if (!gr){
       discreteRegion *dr = new discreteRegion(gm, tag);
-      std::set<int> faces;
+      std::vector<int> faces, signs;
       for (int j = 0; j < n; j++){
 	int tagf;
 	if(fscanf(fp, "%d", &tagf) != 1) return;
-	faces.insert(tagf);
+	faces.push_back(std::abs(tagf));
+        int sign = tagf > 0 ? 1 : -1;
+        signs.push_back(sign);
       }
-      dr->setBoundFaces(faces);
+      dr->setBoundFaces(faces, signs);
       gm->add(dr);
       gr = dr;
     }
@@ -112,38 +117,51 @@ void readMSHPeriodicNodes(FILE *fp, GModel *gm)
   if(fscanf(fp, "%d", &count) != 1) return;
   for(int i = 0; i < count; i++){
     int dim, slave, master;
+
     if(fscanf(fp, "%d %d %d", &dim, &slave, &master) != 3) continue;
+
     GEntity *s = 0, *m = 0;
     switch(dim){
     case 0 : s = gm->getVertexByTag(slave); m = gm->getVertexByTag(master); break;
     case 1 : s = gm->getEdgeByTag(slave);   m = gm->getEdgeByTag(master);   break;
     case 2 : s = gm->getFaceByTag(slave);   m = gm->getFaceByTag(master);   break;
     }
-    if (s && m){
-      char token[6];
-      fpos_t pos;
-      fgetpos(fp, &pos);
-      if(fscanf(fp, "%s", token) != 1) return;
-      if(strcmp(token, "Affine") == 0) {
-        std::vector<double> tfo;
-        for(int i = 0; i < 16; i++){
-          if(fscanf(fp, "%lf", &tfo[i]) != 1) return;
-        }
-        s->setMeshMaster(m, tfo);
-      }
-      else {
-        fsetpos(fp, &pos);
-        s->setMeshMaster(m);
-      }
-      int numv;
-      if(fscanf(fp, "%d", &numv) != 1) numv = 0;
-      for(int j = 0; j < numv; j++){
-        int v1, v2;
-        if(fscanf(fp, "%d %d", &v1, &v2) != 2) continue;
-        MVertex *mv1 = gm->getMeshVertexByTag(v1);
-        MVertex *mv2 = gm->getMeshVertexByTag(v2);
-        s->correspondingVertices[mv1] = mv2;
+
+    // we need to continue parsing, otherwise we end up reading on the wrong position
+
+    bool completePer = s && m;
+
+    char token[6];
+    fpos_t pos;
+    fgetpos(fp, &pos);
+    if(fscanf(fp, "%s", token) != 1) return;
+    if(strcmp(token, "Affine") == 0) {
+      std::vector<double> tfo(16);
+      for(int i = 0; i < 16; i++){
+        if(fscanf(fp, "%lf", &tfo[i]) != 1) return;
       }
+      if(completePer) s->setMeshMaster(m, tfo);
+    }
+    else {
+      fsetpos(fp, &pos);
+      if(completePer) s->setMeshMaster(m);
+    }
+    int numv;
+    if(fscanf(fp, "%d", &numv) != 1) numv = 0;
+    for(int j = 0; j < numv; j++){
+      int v1, v2;
+      if(fscanf(fp, "%d %d", &v1, &v2) != 2) continue;
+      MVertex *mv1 = gm->getMeshVertexByTag(v1);
+      MVertex *mv2 = gm->getMeshVertexByTag(v2);
+      if(completePer) s->correspondingVertices[mv1] = mv2;
+    }
+    if (!completePer) {
+      if (!s)
+        Msg::Info("Could not find periodic slave entity %d of dimension %d",
+                     slave, dim);
+      if (!m)
+        Msg::Info("Could not find periodic master entity %d of dimension %d",
+                  master, dim);
     }
   }
 }
@@ -170,7 +188,7 @@ int GModel::readMSH(const std::string &name)
   double version = 0.;
   bool binary = false, swap = false, postpro = false;
   int minVertex = 0;
-  std::map<int, std::vector<MElement*> > elements[10];
+  std::map<int, std::vector<MElement*> > elements[11];
 
   while(1) {
 
@@ -196,12 +214,12 @@ int GModel::readMSH(const std::string &name)
       }
       if(format){
         binary = true;
-        Msg::Info("Mesh is in binary format");
+        Msg::Debug("Mesh is in binary format");
         int one;
         if(fread(&one, sizeof(int), 1, fp) != 1){ fclose(fp); return 0; }
         if(one != 1){
           swap = true;
-          Msg::Info("Swapping bytes from binary file");
+          Msg::Debug("Swapping bytes from binary file");
         }
       }
     }
@@ -340,7 +358,7 @@ int GModel::readMSH(const std::string &name)
       if((int)_vertexMapCache.size() == numVertices &&
          ((minVertex == 1 && maxVertex == numVertices) ||
           (minVertex == 0 && maxVertex == numVertices - 1))){
-        Msg::Info("Vertex numbering is dense");
+        Msg::Debug("Vertex numbering is dense");
         _vertexVectorCache.resize(_vertexMapCache.size() + 1);
         if(minVertex == 1)
           _vertexVectorCache[0] = 0;
@@ -406,8 +424,9 @@ int GModel::readMSH(const std::string &name)
         case TYPE_HEX: elements[5][entity].push_back(element); break;
         case TYPE_PRI: elements[6][entity].push_back(element); break;
         case TYPE_PYR: elements[7][entity].push_back(element); break;
-        case TYPE_POLYG: elements[8][entity].push_back(element); break;
-        case TYPE_POLYH: elements[9][entity].push_back(element); break;
+        case TYPE_TRIH: elements[8][entity].push_back(element); break;
+        case TYPE_POLYG: elements[9][entity].push_back(element); break;
+        case TYPE_POLYH: elements[10][entity].push_back(element); break;
         }
         if(numElements > 100000)
           Msg::ProgressMeter(i + 1, numElements, true, "Reading elements");
@@ -415,7 +434,7 @@ int GModel::readMSH(const std::string &name)
     }
 
     // $Periodical section
-    else if(!strncmp(&str[1], "Periodical", 10)) {
+    else if(!strncmp(&str[1], "Periodic", 8)) {
       readMSHPeriodicNodes(fp, this);
     }
 
@@ -433,6 +452,8 @@ int GModel::readMSH(const std::string &name)
     } while(str[0] != '$');
   }
 
+
+
   // store the elements in their associated elementary entity. If the
   // entity does not exist, create a new (discrete) one.
   for(int i = 0; i < (int)(sizeof(elements) / sizeof(elements[0])); i++)
@@ -447,6 +468,11 @@ int GModel::readMSH(const std::string &name)
   else
     _storeVerticesInEntities(_vertexMapCache);
 
+  _createGeometryOfDiscreteEntities() ;
+
+  for(int i = 0; i < (int)(sizeof(elements) / sizeof(elements[0])); i++)
+    _storeParentsInSubElements(elements[i]);
+
   fclose(fp);
 
   return postpro ? 2 : 1;
@@ -460,7 +486,7 @@ static void writeMSHPhysicals(FILE *fp, GEntity *ge)
     fprintf(fp, "%d ", *itp);
 }
 
-static void writeMSHEntities(FILE *fp, GModel *gm)
+void writeMSHEntities(FILE *fp, GModel *gm)
 {
   fprintf(fp, "$Entities\n");
   fprintf (fp, "%d %d %d %d\n", gm->getNumVertices(), gm->getNumEdges(),
@@ -483,19 +509,37 @@ static void writeMSHEntities(FILE *fp, GModel *gm)
   }
   for (GModel::fiter it = gm->firstFace(); it != gm->lastFace(); ++it) {
     std::list<GEdge*> edges = (*it)->edges();
+    std::list<int> ori = (*it)->edgeOrientations();
     fprintf(fp, "%d %d ", (*it)->tag(), (int)edges.size());
-    for(std::list<GEdge*>::iterator ite = edges.begin(); ite != edges.end(); ite++){
-      fprintf(fp, "%d ", (*ite)->tag());
+    std::vector<int> tags, signs;
+    for(std::list<GEdge*>::iterator ite = edges.begin(); ite != edges.end(); ite++)
+      tags.push_back((*ite)->tag());
+    for(std::list<int>::iterator ite = ori.begin(); ite != ori.end(); ite++)
+      signs.push_back(*ite);
+    if(tags.size() == signs.size()){
+      for(unsigned int i = 0; i < tags.size(); i++)
+        tags[i] *= (signs[i] > 0 ? 1 : -1);
     }
+    for(unsigned int i = 0; i < tags.size(); i++)
+      fprintf(fp, "%d ", tags[i]);
     writeMSHPhysicals(fp, *it);
     fprintf(fp, "\n");
   }
   for (GModel::riter it = gm->firstRegion(); it != gm->lastRegion(); ++it) {
     std::list<GFace*> faces = (*it)->faces();
+    std::list<int> ori = (*it)->faceOrientations();
     fprintf(fp, "%d %d ", (*it)->tag(), (int)faces.size());
-    for(std::list<GFace*>::iterator itf = faces.begin(); itf != faces.end(); itf++){
-      fprintf(fp, "%d ", (*itf)->tag());
+    std::vector<int> tags, signs;
+    for(std::list<GFace*>::iterator itf = faces.begin(); itf != faces.end(); itf++)
+      tags.push_back((*itf)->tag());
+    for(std::list<int>::iterator itf = ori.begin(); itf != ori.end(); itf++)
+      signs.push_back(*itf);
+    if(tags.size() == signs.size()){
+      for(unsigned int i = 0; i < tags.size(); i++)
+        tags[i] *= (signs[i] > 0 ? 1 : -1);
     }
+    for(unsigned int i = 0; i < tags.size(); i++)
+      fprintf(fp, "%d ", tags[i]);
     writeMSHPhysicals(fp, *it);
     fprintf(fp, "\n");
   }
@@ -546,7 +590,7 @@ static void writeElementsMSH(FILE *fp, GModel *model, GEntity *ge, std::vector<T
   }
 }
 
-void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity*> &entities)
+void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity*> &entities, bool renumber)
 {
   int count = 0;
   for (unsigned int i = 0; i < entities.size(); i++)
@@ -571,7 +615,10 @@ void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity*> &entities)
            it != g_slave->correspondingVertices.end(); it++){
         MVertex *v1 = it->first;
         MVertex *v2 = it->second;
-        fprintf(fp, "%d %d\n", v1->getIndex(), v2->getIndex());
+        if(renumber)
+          fprintf(fp, "%d %d\n", v1->getIndex(), v2->getIndex());
+        else
+          fprintf(fp, "%d %d\n", v1->getNum(), v2->getNum());
       }
     }
   }
@@ -665,6 +712,9 @@ int GModel::writeMSH(const std::string &name, double version, bool binary,
   for(riter it = firstRegion(); it != lastRegion(); ++it)
     writeElementsMSH(fp, this, *it, (*it)->pyramids, saveAll, saveSinglePartition,
                      binary);
+  for(riter it = firstRegion(); it != lastRegion(); ++it)
+    writeElementsMSH(fp, this, *it, (*it)->trihedra, saveAll, saveSinglePartition,
+                     binary);
   for(fiter it = firstFace(); it != lastFace(); ++it)
     writeElementsMSH(fp, this, *it, (*it)->triangles, saveAll, saveSinglePartition,
                      binary);
@@ -683,7 +733,7 @@ int GModel::writeMSH(const std::string &name, double version, bool binary,
   fprintf(fp, "$EndElements\n");
 
   //save periodic nodes
-  writeMSHPeriodicNodes(fp, entities);
+  writeMSHPeriodicNodes(fp, entities, renumber);
 
   fclose(fp);
 
